package com.xzzn.ems.service.impl;

import com.xzzn.common.enums.DeviceCategory;
import com.xzzn.ems.domain.EmsSiteSetting;
import com.xzzn.ems.domain.vo.SiteTotalInfoVo;
import com.xzzn.ems.mapper.EmsDevicesSettingMapper;
import com.xzzn.ems.mapper.EmsSiteSettingMapper;
import com.xzzn.ems.service.IEmsSiteService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;

/**
 * 站点信息 服务层实现
 *
 */
@Service
public class EmsSiteServiceImpl implements IEmsSiteService
{

    @Autowired
    private EmsSiteSettingMapper emsSiteMapper;
    @Autowired
    private EmsDevicesSettingMapper emsDevicesMapper;

    @Override
    public List<EmsSiteSetting> getAllSites() {
        return emsSiteMapper.selectEmsSiteSettingList(null);
    }

    @Override
    public SiteTotalInfoVo getSiteTotalInfo() {
        return emsSiteMapper.getSiteTotalInfo();
    }

    /**
     * 根据site_id获取所有电池堆
     * @param siteId
     * @return
     */
    @Override
    public List<Map<String, Object>> getAllStackInfo(String siteId) {
        return emsDevicesMapper.getDeviceInfosBySiteIdAndCategory(siteId, DeviceCategory.STACK.getCode());
    }

    /**
     * 根据父类设备id获取子设备id
     * @param parentId
     * @return
     */
    @Override
    public List<Map<String, Object>> getAllClusterInfo(String parentId) {
        return emsDevicesMapper.getDeviceInfoByParentId(parentId);
    }

    /**
     * 获取站点列表
     * @param siteName
     * @param startTime
     * @param endTime
     * @return
     */
    @Override
    public List<EmsSiteSetting> getAllSiteInfoList(String siteName, String startTime, String endTime) {
        return emsSiteMapper.getSiteInfoList(siteName,startTime,endTime);
    }

}
