package com.xzzn.ems.service.impl;

import com.xzzn.common.utils.DateUtils;
import com.xzzn.common.utils.StringUtils;
import com.xzzn.ems.domain.EmsSiteSetting;
import com.xzzn.ems.domain.vo.*;
import com.xzzn.ems.mapper.EmsAlarmRecordsMapper;
import com.xzzn.ems.mapper.EmsPcsDataMapper;
import com.xzzn.ems.mapper.EmsSiteSettingMapper;
import com.xzzn.ems.service.IEmsSiteService;
import com.xzzn.ems.service.IHomePageService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.*;

/**
 * 首页看板+站点地图 服务层实现
 *
 */
@Service
public class HomePageServiceImpl implements IHomePageService
{
    private static final int LIMIT_TIME = 6;

    @Autowired
    private IEmsSiteService emsSiteService;
    @Autowired
    private EmsPcsDataMapper emsPcsDataMapper;
    @Autowired
    private EmsSiteSettingMapper emsSiteMapper;
    @Autowired
    private EmsAlarmRecordsMapper alarmRecordsMapper;

    @Override
    public SiteTotalInfoVo getSiteTotalInfo() {
        SiteTotalInfoVo siteTotalInfoVo = new SiteTotalInfoVo();
        siteTotalInfoVo = emsSiteService.getSiteTotalInfo();
        // 获取总充+总放
        Map<String,BigDecimal> pcsMap  = emsPcsDataMapper.getPcsTotalChargeData(null);
        if(pcsMap != null){
            siteTotalInfoVo.setTotalChargedCap(pcsMap.get("totalChargedCap"));
            siteTotalInfoVo.setTotalDischargedCap(pcsMap.get("totalDischargedCap"));
        }
        return siteTotalInfoVo;
    }

    // 单站点基本信息
    @Override
    public SingleSiteBaseInfo getSingleSiteBaseInfo(String siteId) {
        SingleSiteBaseInfo singleSiteBaseInfo = new SingleSiteBaseInfo();

        if (!StringUtils.isEmpty(siteId)) {
            // 站点基本信息
            EmsSiteSetting emsSite = emsSiteMapper.selectEmsSiteSettingBySiteId(siteId);
            if (emsSite != null) {
                // 装机功率+装机容量
                singleSiteBaseInfo.setSiteName(emsSite.getSiteName());
                singleSiteBaseInfo.setInstallCapacity(emsSite.getInstallCapacity());
                singleSiteBaseInfo.setInstallPower(emsSite.getInstallPower());
                String[] siteLocation = new String[2];
                siteLocation[0] = emsSite.getLongitude() == null ? ""  : emsSite.getLongitude().toString();
                siteLocation[1] = emsSite.getLatitude() ==  null ? ""  : emsSite.getLatitude().toString();
                singleSiteBaseInfo.setSiteLocation(siteLocation);//站点位置
                singleSiteBaseInfo.setSiteAddress(emsSite.getSiteAddress());
                singleSiteBaseInfo.setRunningTime(emsSite.getRunningTime() == null ? null :
                        DateUtils.parseDateToStr("yyyy-MM-dd",emsSite.getRunningTime()));//投运时间
                // 获取单站点的总充+总放
                Map<String,BigDecimal> pcsMap  = emsPcsDataMapper.getPcsTotalChargeData(siteId);
                if (pcsMap != null) {
                    singleSiteBaseInfo.setTotalChargedCap(pcsMap.get("totalChargedCap"));
                    singleSiteBaseInfo.setTotalDisChargedCap(pcsMap.get("totalDischargedCap"));
                }
                // 七天放电数据统计
                List<SiteMonitorDataVo> siteMonitorDataVoList = emsPcsDataMapper.getPcsDataBySiteId(siteId,LIMIT_TIME);
                singleSiteBaseInfo.setSevenDayDisChargeStats(siteMonitorDataVoList);
                // 充放电基本数据处理
                dealSitePCSDate(singleSiteBaseInfo,siteMonitorDataVoList);
            }
        }

        return singleSiteBaseInfo;
    }

    private void dealSitePCSDate(SingleSiteBaseInfo singleSiteBaseInfo, List<SiteMonitorDataVo> siteMonitorDataVoList) {
        if (siteMonitorDataVoList != null && !siteMonitorDataVoList.isEmpty()) {
            for (SiteMonitorDataVo sitePcsData : siteMonitorDataVoList) {
                // 获取当天的充电量+放电量
                String pcsDate = sitePcsData.getAmmeterDate();
                boolean isToday= checkIsToday(pcsDate);
                if(isToday){
                    singleSiteBaseInfo.setDayChargedCap(sitePcsData.getChargedCap());
                    singleSiteBaseInfo.setDayDisChargedCap(sitePcsData.getDisChargedCap());
                }
            }
        }
    }

    private boolean checkIsToday(String pcsDate) {
        boolean flag = false;
        Calendar calendar = Calendar.getInstance();
        int todayMonth = calendar.get(Calendar.MONTH) + 1;
        int todayDay = calendar.get(Calendar.DAY_OF_MONTH);
        if (StringUtils.isNotEmpty(pcsDate)){
            String[] pcsDateArray = pcsDate.split("/");
            if (todayMonth == Integer.parseInt(pcsDateArray[0]) &&
                    todayDay == Integer.parseInt(pcsDateArray[1])) {
                flag = true;
            }
        }
        return flag;
    }

    /**
     * 首页看板-数据概览
     */
    @Override
    public HomePageDataViewVo getHomePageDataList() {
        HomePageDataViewVo  homePageDataViewVo = new HomePageDataViewVo();
        // 电量指标
        List<ElectricIndexList> electricDataList = emsPcsDataMapper.getElectDataList();
        homePageDataViewVo.setElecDataList(electricDataList);
        // 系统效率
        // 告警趋势
        List<AlarmTrendList> alarmTrendList = alarmRecordsMapper.getAlarmTrendList();
        homePageDataViewVo.setAlarmDataList(alarmTrendList);
        // 设备告警占比
        List<DeviceAlarmProportionList> deviceAlarmPropList = alarmRecordsMapper.getDeviceAlarmPropList();
        homePageDataViewVo.setDeviceAlarmList(deviceAlarmPropList);
        // 告警等级分布
        return homePageDataViewVo;
    }

}
