package com.xzzn.ems.service.impl;

import com.xzzn.common.constant.RedisKeyConstants;
import com.xzzn.common.core.redis.RedisCache;
import com.xzzn.common.enums.DeviceCategory;
import com.xzzn.common.utils.StringUtils;
import com.xzzn.ems.domain.*;
import com.xzzn.ems.domain.vo.*;
import com.xzzn.ems.mapper.*;
import com.xzzn.ems.service.ISingleSiteService;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.util.*;

/**
 * 单点监控 服务层实现
 */
@Service
public class SingleSiteServiceImpl implements ISingleSiteService {

    private static final String CLUSTER_DATA_TEP = "温度";

    private static final String CLUSTER_DATA_VOLTAGE = "电压";

    private static final String CLUSTER_DATA_SOC = "SOC";

    @Autowired
    private EmsPcsDataMapper emsPcsDataMapper;
    @Autowired
    private EmsAlarmRecordsMapper emsAlarmRecordsMapper;
    @Autowired
    private EmsBatteryDataMapper emsBatteryDataMapper;
    @Autowired
    private EmsBatteryStackMapper emsBatteryStackMapper;
    @Autowired
    private EmsBatteryClusterMapper emsBatteryClusterMapper;
    @Autowired
    private EmsPcsBranchDataMapper emsPcsBranchDataMapper;
    @Autowired
    private EmsCoolingDataMapper emsCoolingDataMapper;
    @Autowired
    private EmsAmmeterDataMapper emsAmmeterDataMapper;
    @Autowired
    private EmsDevicesSettingMapper emsDevicesSettingMapper;

    @Autowired
    private RedisCache redisCache;

    @Override
    public SiteMonitorHomeVo getSiteMonitorDataVo(String siteId) {
        SiteMonitorHomeVo siteMonitorHomeVo = new SiteMonitorHomeVo();

        if (!StringUtils.isEmpty(siteId)) {
            // 获取单站点的总充+总放+电网实时功率
            Map<String,BigDecimal> pcsMap  = emsPcsDataMapper.getPcsTotalChargeData(siteId);
            if (pcsMap != null) {
                siteMonitorHomeVo.setTotalChargedCap(pcsMap.get("totalChargedCap"));
                siteMonitorHomeVo.setTotalDischargedCap(pcsMap.get("totalDischargedCap"));
                siteMonitorHomeVo.setGridNrtPower(pcsMap.get("gridNrtPower"));
            }
            // 实时告警数据 名称+状态+告警内容
            List<SiteMonitorHomeAlarmVo> siteMonitorHomeAlarmVo = emsAlarmRecordsMapper.getAlarmRecordsBySiteId(siteId);
            siteMonitorHomeVo.setSiteMonitorHomeAlarmVo(siteMonitorHomeAlarmVo);
            // 能量数据
            List<SiteMonitorDataVo> siteMonitorDataVoList = emsPcsDataMapper.getPcsDataBySiteId(siteId,6);
            if (!CollectionUtils.isEmpty(siteMonitorDataVoList)) {
                for (SiteMonitorDataVo sitePcsData : siteMonitorDataVoList) {
                    // 获取当天的充电量+放电量
                    String pcsDate = sitePcsData.getAmmeterDate();
                    boolean isToday= checkIsToday(pcsDate);
                    if(isToday){
                        siteMonitorHomeVo.setDayChargedCap(sitePcsData.getChargedCap());
                        siteMonitorHomeVo.setDayDisChargedCap(sitePcsData.getDisChargedCap());
                    }
                }
                // 储能可用电量
                BigDecimal energyStorageAvailElec = siteMonitorHomeVo.getTotalDischargedCap().subtract(siteMonitorHomeVo.getTotalChargedCap());
                siteMonitorHomeVo.setEnergyStorageAvailElec(energyStorageAvailElec);
            }
            siteMonitorHomeVo.setSiteMonitorDataVo(siteMonitorDataVoList);
        }

        return siteMonitorHomeVo;
    }

    private boolean checkIsToday(String pcsDate) {
        boolean flag = false;
        Calendar calendar = Calendar.getInstance();
        int todayMonth = calendar.get(Calendar.MONTH) + 1;
        int todayDay = calendar.get(Calendar.DAY_OF_MONTH);
        if (StringUtils.isNotEmpty(pcsDate)){
            String[] pcsDateArray = pcsDate.split("/");
            if (todayMonth == Integer.parseInt(pcsDateArray[0]) &&
                    todayDay == Integer.parseInt(pcsDateArray[1])) {
                flag = true;
            }
        }
        return flag;
    }

    // 获取单站监控实时运行头部数据
    @Override
    public SiteMonitorRunningHeadInfoVo getSiteRunningHeadInfo(String siteId) {
        SiteMonitorRunningHeadInfoVo siteMonitorRunningHeadInfoVo = new SiteMonitorRunningHeadInfoVo();

        if (!StringUtils.isEmpty(siteId)) {
            // 实时有功功率/实时无功功率/今日充电量/今日放电量
            siteMonitorRunningHeadInfoVo = emsPcsDataMapper.getSiteRunningHeadInfo(siteId);
            // 电池簇SOC
            // 电池堆SOH
            EmsBatteryData emsBatteryData = emsBatteryDataMapper.getBatteryDataBySiteId(siteId);
            if (emsBatteryData != null) {
                siteMonitorRunningHeadInfoVo.setSoc(emsBatteryData.getSoc());
                siteMonitorRunningHeadInfoVo.setSoh(emsBatteryData.getSoh());
            }
        }

        return siteMonitorRunningHeadInfoVo;
    }

    //  获取单站监控实时运行曲线图数据
    @Override
    public SiteMonitorRuningInfoVo getRunningGraph(String siteId) {
        SiteMonitorRuningInfoVo siteMonitorRuningInfoVo = new SiteMonitorRuningInfoVo();
        if (!StringUtils.isEmpty(siteId)) {
            //储能功率list
            //pcs平均温度list
            //电池平均soclist
            //电池平均温度list
        }
        return siteMonitorRuningInfoVo;
    }

    // 根据site_id获取pcs详细数据+支路数据
    @Override
    public List<PcsDetailInfoVo> getPcsDetailInfo(String siteId) {
        List<PcsDetailInfoVo> pcsDetailInfoVoList = new ArrayList<>();

        if (!StringUtils.isEmpty(siteId)) {
            // 获取该设备下所有pcs的id
            List<Map<String, Object>> pcsIds = emsDevicesSettingMapper.getDeviceInfosBySiteIdAndCategory(siteId, DeviceCategory.PCS.getCode());

            for (Map<String, Object> pcsDevice : pcsIds) {
                PcsDetailInfoVo pcsDetailInfoVo = new PcsDetailInfoVo();
                pcsDetailInfoVo.setDeviceName(pcsDevice.get("deviceName").toString());
                pcsDetailInfoVo.setCommunicationStatus(pcsDevice.get("communicationStatus").toString());
                // 从redis取pcs单个详细数据
                String pcsId = pcsDevice.get("id").toString();
                EmsPcsData pcsData = redisCache.getCacheObject(RedisKeyConstants.PCS +siteId+"_"+pcsId);
                if  (pcsData != null) {
                    BeanUtils.copyProperties(pcsData, pcsDetailInfoVo);
                }
                // 支路信息数据
                List<PcsBranchInfo> pcsBranchInfoList  = new  ArrayList<>();
                processBranchDataInfo(siteId,pcsId,pcsBranchInfoList);
                pcsDetailInfoVo.setPcsBranchInfoList(pcsBranchInfoList);

                pcsDetailInfoVoList.add(pcsDetailInfoVo);
            }
        }
        return pcsDetailInfoVoList;
    }

    private void processBranchDataInfo(String siteId, String pcsId, List<PcsBranchInfo> pcsBranchInfoList) {
        if (!StringUtils.isEmpty(pcsId)) {
            List<EmsPcsBranchData> pcsBranchData = redisCache.getCacheObject(RedisKeyConstants.BRANCH +siteId+"_"+pcsId);
            if (pcsBranchData != null) {
                for(EmsPcsBranchData emsPcsBranchData : pcsBranchData) {
                    PcsBranchInfo pcsBranchInfo = new PcsBranchInfo();
                    BeanUtils.copyProperties(emsPcsBranchData, pcsBranchInfo);
                    pcsBranchInfoList.add(pcsBranchInfo);
                }
            }
        }
    }

    // 获取BMS总览数据
    @Override
    public List<BMSOverViewVo> getBMSOverView(String siteId) {
        List<BMSOverViewVo> bmsOverViewVoList = new ArrayList<>();

        // 获取所有电池堆
        if (!StringUtils.isEmpty(siteId)) {
            List<Map<String, Object>> stackIds = emsDevicesSettingMapper.getDeviceInfosBySiteIdAndCategory(siteId, DeviceCategory.STACK.getCode());
            for (Map<String, Object> stackDevice : stackIds) {
                BMSOverViewVo bmsOverViewVo = new BMSOverViewVo();
                bmsOverViewVo.setDeviceName(stackDevice.get("deviceName").toString());
                // 从redis取堆单个详细数据
                String stackId = stackDevice.get("id").toString();
                EmsBatteryStack stackData = redisCache.getCacheObject(RedisKeyConstants.STACK +siteId+"_"+stackId);
                if  (stackData != null) {
                    BeanUtils.copyProperties(stackData, bmsOverViewVo);
                }
                // 列表数据
                if (!StringUtils.isEmpty(stackId)) {
                    List<BMSBatteryDataList> batteryDataList = emsBatteryClusterMapper.getBmsBatteryData(siteId,stackId);
                    bmsOverViewVo.setBatteryDataList(batteryDataList);
                }
                bmsOverViewVoList.add(bmsOverViewVo);
            }
        }
        return bmsOverViewVoList;
    }

    // 获取BMS电池簇数据
    @Override
    public List<BMSBatteryClusterVo> getBMSBatteryCluster(String siteId) {
        List<BMSBatteryClusterVo> bmsBatteryClusterVoList = new ArrayList<>();

        if (!StringUtils.isEmpty(siteId)) {
            // 获取所有设备下的电池簇id
            List<Map<String, Object>> clusterIds = emsDevicesSettingMapper.getDeviceInfosBySiteIdAndCategory(siteId, DeviceCategory.CLUSTER.getCode());
            for (Map<String, Object> clusterDevice : clusterIds) {
                BMSBatteryClusterVo bmsBatteryClusterVo = new BMSBatteryClusterVo();
                bmsBatteryClusterVo.setDeviceName(clusterDevice.get("deviceName").toString());
                // 从redis取单个簇详细数据
                String clusterId = clusterDevice.get("id").toString();
                EmsBatteryCluster clusterData = redisCache.getCacheObject(RedisKeyConstants.CLUSTER +siteId+"_"+clusterId);
                if  (clusterData != null) {
                    BeanUtils.copyProperties(clusterData, bmsBatteryClusterVo);
                    // 处理单体电池数据-平均/最大/最小
                    List<BMSBatteryClusterDataList> clusterDataList = new ArrayList<>();
                    dealWithBatteryClusterData(clusterData,clusterDataList);
                    bmsBatteryClusterVo.setBatteryDataList(clusterDataList);
                }

                bmsBatteryClusterVoList.add(bmsBatteryClusterVo);
            }
        }
        return bmsBatteryClusterVoList;
    }

    private void dealWithBatteryClusterData(EmsBatteryCluster clusterData, List<BMSBatteryClusterDataList> clusterDataList) {
        BMSBatteryClusterDataList voltageData = new BMSBatteryClusterDataList();
        BMSBatteryClusterDataList tempData = new BMSBatteryClusterDataList();
        BMSBatteryClusterDataList socData = new BMSBatteryClusterDataList();

        // 设值
        voltageData.setDataName(CLUSTER_DATA_VOLTAGE);
        voltageData.setAvgData(clusterData.getAvgCellVoltage());
        voltageData.setMaxData(clusterData.getMaxCellVoltage());
        voltageData.setMaxDataID(clusterData.getMaxCellVoltageId().toString());
        voltageData.setMinData(clusterData.getMinCellVoltage());
        voltageData.setMinDataID(clusterData.getMinCellVoltageId().toString());
        tempData.setDataName(CLUSTER_DATA_TEP);
        tempData.setAvgData(clusterData.getAvgCellTemp());
        tempData.setMaxData(clusterData.getMaxCellTemp());
        tempData.setMaxDataID(clusterData.getMaxCellTempId().toString());
        tempData.setMinData(clusterData.getMinCellTemp());
        tempData.setMinDataID(clusterData.getMinCellTempId().toString());
        socData.setDataName(CLUSTER_DATA_SOC);
        socData.setAvgData(clusterData.getCurrentSoc());
        socData.setMaxData(clusterData.getMaxCellSoc());
        socData.setMaxDataID(clusterData.getMaxCellSocId().toString());
        socData.setMinData(clusterData.getMinCellSoc());
        socData.setMinDataID(clusterData.getMinCellSocId().toString());

        clusterDataList.add(voltageData);
        clusterDataList.add(tempData);
        clusterDataList.add(socData);
    }

    // 获取液冷设备参数
    @Override
    public  List<EmsCoolingData> getCoolingDataList(String siteId) {
        List<EmsCoolingData> emsCoolingDataList = new ArrayList<>();
        if (!StringUtils.isEmpty(siteId)) {
            emsCoolingDataList = emsCoolingDataMapper.getCoolingDataList(siteId);
        }
        return emsCoolingDataList;
    }


    /**
     * 根据电池堆获取单体电池数据
     * @param clusterDeviceId
     * @return
     */
    @Override
    public  List<BatteryDataStatsListVo> getClusterDataInfoList(String clusterDeviceId,String  siteId) {
        List<BatteryDataStatsListVo> batteryDataStatsListVo = new ArrayList<>();
        List<Map<String, Object>> clusterIds = new ArrayList<>();
        if (StringUtils.isEmpty(clusterDeviceId)) {
            clusterIds = emsDevicesSettingMapper.getDeviceInfosBySiteIdAndCategory(siteId, DeviceCategory.CLUSTER.getCode());
        } else {
            Map<String, Object> map = new HashMap<>();
            map.put("id", clusterDeviceId);
            clusterIds.add(map);
        }
        for (Map<String, Object> clusterDevice : clusterIds) {
            // 从redis取单个簇详细数据
            String clusterId = clusterDevice.get("id").toString();
            List<EmsBatteryData> batteryDataList = redisCache.getCacheList(RedisKeyConstants.BATTERY + siteId + "_" + clusterId);
            if (batteryDataList != null) {
               for (EmsBatteryData batteryData : batteryDataList) {
                   BatteryDataStatsListVo batteryDataStatsVo = new BatteryDataStatsListVo();
                   BeanUtils.copyProperties(batteryData, batteryDataStatsVo);
                   batteryDataStatsListVo.add(batteryDataStatsVo);
               }
            }
        }
        return batteryDataStatsListVo;
    }

    /**
     * 电表数据信息
     * @param siteId
     * @return
     */
    @Override
    public List<AmmeterDataVo> getAmmeterDataList(String siteId) {
        List<AmmeterDataVo> ammeterDataVos = new ArrayList<>();

        if (!StringUtils.isEmpty(siteId)) {
            // 先获取所有电表设备
            List<EmsDevicesSetting> devicesList = emsDevicesSettingMapper.getAllBatteryDeviceBySiteId(siteId);
            if (!CollectionUtils.isEmpty(devicesList)) {
                for (EmsDevicesSetting devicesSetting : devicesList) {
                    AmmeterDataVo ammeterDataVo = new AmmeterDataVo();
                    ammeterDataVo.setDeviceName(devicesSetting.getDeviceName());
                    ammeterDataVo.setEmsCommunicationStatus(devicesSetting.getCommunicationStatus());
                    String deviceId = devicesSetting.getDeviceId();
                    // 获取类别数据
                    List<AmmeterDataDetailInfo> ammeterDataDetailInfos = emsAmmeterDataMapper.getAmmeterDetailInfo(siteId,deviceId);
                    ammeterDataVo.setAmmeterDataDetailInfos(ammeterDataDetailInfos);
                    // 数据更新时间
                    ammeterDataVo.setDataUpdateTime(ammeterDataDetailInfos.get(0).getUpdateTime());

                    ammeterDataVos.add(ammeterDataVo);
                }
            }
        }
        return ammeterDataVos;
    }

}
